@@ -3,6 +3,7 @@
 
 #include <QtGui>
 #include "puzzleitem.h"
+#include "jigsawpuzzleboard.h"
 
 class JigsawPuzzleBoard;
 
@@ -11,21 +12,18 @@ class JigsawPuzzleItem : public QObject, public PuzzleItem
     Q_OBJECT
     Q_PROPERTY(QPointF pos READ pos WRITE setPos)
     QPointF _dragStart;
-    QSize _unit;
     bool _dragging;
     bool _canMerge;
-    int _tolerance;
     double _weight;
 
 public:
-    explicit JigsawPuzzleItem(const QPixmap &pixmap, const QSize &unitSize, QGraphicsItem *parent = 0, QGraphicsScene *scene = 0);
+    explicit JigsawPuzzleItem(const QPixmap &pixmap, QGraphicsItem *parent = 0, QGraphicsScene *scene = 0);
     virtual bool merge(JigsawPuzzleItem *item);
     bool canMerge() const;
     void setMerge(bool canMerge);
     double weight();
-    void setTolerance(int t);
-    int tolerance();
-    const QSize &unit();
+    inline int tolerance();
+    inline const QSize &unit();
     void raise();
     void verifyPosition();
     void verifyCoveredSiblings();
@@ -44,4 +42,14 @@ protected:
 
 };
 
+inline const QSize &JigsawPuzzleItem::unit()
+{
+    return ((JigsawPuzzleBoard*)scene())->unit();
+}
+
+inline int JigsawPuzzleItem::tolerance()
+{
+    return ((JigsawPuzzleBoard*)scene())->tolerance();
+}
+
 #endif // JIGSAWPUZZLEITEM_H
