@@ -14,17 +14,25 @@
 #include <QGLWidget>
 #endif
 
+#if defined(Q_OS_SYMBIAN)
+#include <eikenv.h>
+#include <eikappui.h>
+#include <aknenv.h>
+#include <aknappui.h>
+#endif
+
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
     ui(new Ui::MainWindow),
     timer(new QTimer(this)),
     settings(new SettingsDialog(this)),
-    chooser(new ImageChooser(this)),
+    newgame(new NewGameDialog(this)),
     highscores(new HighScoresDialog(this)),
     board(new JigsawPuzzleBoard(this)),
     _isPlaying(false),
     _isPaused(false),
     _wasPaused(false),
+    _canPause(false),
     _currentScaleRatio(1)
 {
     ui->setupUi(this);
@@ -34,6 +42,7 @@ MainWindow::MainWindow(QWidget *parent) :
     ui->btnSurrender->hide();
     ui->actionSurrender->setVisible(false);
     ui->btnHelp->hide();
+    ui->btnExit->hide();
 
     timer->setInterval(1000);
     connect(timer, SIGNAL(timeout()), this, SLOT(updateElapsedTimeLabel()));
@@ -56,7 +65,7 @@ MainWindow::MainWindow(QWidget *parent) :
     ui->graphicsView->setOptimizationFlag(QGraphicsView::DontSavePainterState);
     ui->graphicsView->setViewportUpdateMode(QGraphicsView::SmartViewportUpdate);
 
-    delete ui->lblTime;
+    ui->lblTime->hide();
     ui->horizontalLayout->setContentsMargins(10, 0, 10, 0);
 #endif
 
@@ -65,6 +74,12 @@ MainWindow::MainWindow(QWidget *parent) :
     setAttribute(Qt::WA_Maemo5AutoOrientation);
     ui->btnFullscreen->setIcon(QIcon::fromTheme("general_fullsize"));
 #endif
+
+#if defined(Q_OS_SYMBIAN)
+    ui->btnFullscreen->hide();
+    ui->lblTime->show();
+    ui->btnExit->show();
+#endif
 }
 
 MainWindow::~MainWindow()
@@ -75,11 +90,11 @@ MainWindow::~MainWindow()
 bool MainWindow::event(QEvent *event)
 {
     bool result = QMainWindow::event(event);
-    if (_isPlaying && !_wasPaused && _isPaused && event->type() == QEvent::WindowActivate)
+    if (_canPause && _isPlaying && !_wasPaused && _isPaused && event->type() == QEvent::WindowActivate)
     {
         unpause();
     }
-    else if (_isPlaying && event->type() == QEvent::WindowDeactivate)
+    else if (_canPause && _isPlaying && event->type() == QEvent::WindowDeactivate)
     {
         // If the game was paused before deactivating, it will be paused after activating.
         _wasPaused = _isPaused;
@@ -93,7 +108,7 @@ void MainWindow::resizeEvent(QResizeEvent *event)
     if (event)
         QMainWindow::resizeEvent(event);
 
-#if defined(MOBILE)
+#if defined(MOBILE) && !defined(Q_OS_SYMBIAN)
     if (isFullScreen())
     {
         ui->horizontalLayout->removeWidget(ui->btnFullscreen);
@@ -102,27 +117,49 @@ void MainWindow::resizeEvent(QResizeEvent *event)
     }
 #endif
 
+#if defined(Q_OS_SYMBIAN)
+    if (height() > width())
+    {
+        ui->verticalLayout->insertWidget(0, ui->lblTime);
+        ui->btnAbout->hide();
+    }
+    else
+    {
+        ui->horizontalLayout->insertWidget(0, ui->lblTime);
+        ui->btnAbout->show();
+    }
+#endif
+
     if (intro != 0)
         return;
 
     // Scaling the graphics view's content
 
-    // Calculating scale ratio
-    qreal oldScaleRatio = _currentScaleRatio;
-    QSize scaledPixmapSize = board->originalPixmapSize();
-    scaledPixmapSize.scale(ui->graphicsView->size(), Qt::KeepAspectRatio);
-    _currentScaleRatio = min<qreal>((qreal)scaledPixmapSize.width() / (qreal)board->originalPixmapSize().width(),
-                                    (qreal)scaledPixmapSize.height() / (qreal)board->originalPixmapSize().height());
-
-    //qDebug() << "new size" << ui->graphicsView->size() << "ratio" << _currentScaleRatio;
-
-    // Setting scene rect and scale
     QSizeF oldscenesize = board->sceneRect().size();
-    board->setSceneRect(0, 0, ui->graphicsView->width() / _currentScaleRatio, ui->graphicsView->height() / _currentScaleRatio);
-    if (0.98 < _currentScaleRatio && 1.02 > _currentScaleRatio)
-        ui->graphicsView->resetTransform();
+
+    if (SettingsDialog::enableScaling())
+    {
+        // Calculating scale ratio
+        qreal oldScaleRatio = _currentScaleRatio;
+        QSize scaledPixmapSize = board->originalPixmapSize();
+        scaledPixmapSize.scale(ui->graphicsView->size(), Qt::KeepAspectRatio);
+        _currentScaleRatio = min<qreal>((qreal)scaledPixmapSize.width() / (qreal)board->originalPixmapSize().width(),
+                                        (qreal)scaledPixmapSize.height() / (qreal)board->originalPixmapSize().height());
+
+        //qDebug() << "new size" << ui->graphicsView->size() << "ratio" << _currentScaleRatio;
+
+        // Setting scene rect and scale
+        board->setSceneRect(0, 0, ui->graphicsView->width() / _currentScaleRatio, ui->graphicsView->height() / _currentScaleRatio);
+        if (0.98 < _currentScaleRatio && 1.02 > _currentScaleRatio)
+            ui->graphicsView->resetTransform();
+        else
+            ui->graphicsView->scale(_currentScaleRatio / oldScaleRatio, _currentScaleRatio / oldScaleRatio);
+    }
     else
-        ui->graphicsView->scale(_currentScaleRatio / oldScaleRatio, _currentScaleRatio / oldScaleRatio);
+    {
+        ui->graphicsView->resetTransform();
+        board->setSceneRect(0, 0, ui->graphicsView->width(), ui->graphicsView->height());
+    }
 
     // Making sure every piece is visible and has a nice position
     QSizeF p = (board->sceneRect().size() - oldscenesize) / 2;
@@ -152,21 +189,28 @@ void MainWindow::newGame()
 {
     if (!_isPlaying)
     {
-        int rows = SettingsDialog::rows();
-        int cols = SettingsDialog::columns();
-
-        if (!QDialog::Accepted == chooser->exec())
+        if (!QDialog::Accepted == newgame->exec())
             return;
 
+        int rows = SettingsDialog::rows(), cols = SettingsDialog::columns();
         _isPlaying = true;
+        _canPause = false;
+
+        if ((SettingsDialog::startInFullscreen() && !isFullScreen()))// || !SettingsDialog::startInFullscreen() && isFullScreen())
+            toggleFullscreen();
+
+        applyViewportSettings();
 
         QPixmap pixmap;
-        if (pixmap.load(chooser->getPictureFile()) && !pixmap.isNull())
+        if (pixmap.load(newgame->getPictureFile()) && !pixmap.isNull())
         {
             ui->btnOpenImage->hide();
             ui->actionNew_game->setVisible(false);
-            ui->btnSurrender->show();
-            ui->actionSurrender->setVisible(true);
+            if (!isFullScreen())
+            {
+                ui->btnSurrender->show();
+                ui->actionSurrender->setVisible(true);
+            }
 
             if (SettingsDialog::useAccelerometer())
                 fixCurrentOrientation();
@@ -226,6 +270,21 @@ void MainWindow::surrender()
     }
 }
 
+void MainWindow::applyViewportSettings()
+{
+#if defined(HAVE_OPENGL)
+    if (SettingsDialog::useOpenGl() && !ui->graphicsView->viewport()->inherits("QGLWidget"))
+    {
+        ui->graphicsView->viewport()->deleteLater();
+        ui->graphicsView->setViewport(new QGLWidget(QGLFormat(QGL::DoubleBuffer), this));
+    }
+    else if (!SettingsDialog::useOpenGl() && ui->graphicsView->viewport()->inherits("QGLWidget"))
+    {
+        ui->graphicsView->viewport()->deleteLater();
+        ui->graphicsView->setViewport(new QWidget(this));
+    }
+#endif
+}
 
 void MainWindow::onWon()
 {
@@ -245,20 +304,23 @@ void MainWindow::onWon()
 
 void MainWindow::initializeGame()
 {
-    timer->start();
     _secsElapsed = 0;
     updateElapsedTimeLabel();
+    _canPause = true;
 
-    unpause(); // This is also to set the correct icon on the button
+    // This is to set the correct icon on the button and to start the timer
+    unpause();
 
-#if defined (MOBILE)
+#if defined (MOBILE) && !defined(Q_OS_SYMBIAN)
     if (!isFullScreen())
     {
         ui->btnPause->show();
+        ui->btnSurrender->show();
         ui->actionPause->setVisible(true);
     }
 #else
     ui->btnPause->show();
+    ui->btnSurrender->show();
     ui->actionPause->setVisible(true);
 #endif
 
@@ -273,7 +335,7 @@ void MainWindow::endGame()
     timer->stop();
 
     // Additional things
-#if defined(MOBILE)
+#if defined(MOBILE) && !defined(Q_OS_SYMBIAN)
     if (!isFullScreen())
 #endif
         ui->btnOpenImage->show();
@@ -327,6 +389,9 @@ void MainWindow::showSettings()
         board->disableAccelerometer();
         unfixCurrentOrientation();
     }
+
+    //Viewport
+    applyViewportSettings();
 }
 
 void MainWindow::about()
@@ -353,8 +418,8 @@ void MainWindow::updateElapsedTimeLabel()
     else if (_isPlaying)
     {
         _secsElapsed = 0;
-#ifndef MOBILE
         ui->lblTime->setText("Elapsed 0 second");
+#if defined(Q_OS_SYMBIAN) || !defined(MOBILE)
         ui->lblTime->show();
 #endif
     }
@@ -362,12 +427,12 @@ void MainWindow::updateElapsedTimeLabel()
     QString str = "Elapsed " + QString::number(_secsElapsed) + " second";
     if (_secsElapsed > 1)
         str += "s";
-#ifndef MOBILE
+#if defined(Q_OS_SYMBIAN) || !defined(MOBILE)
     if (_isPaused)
         str = "Game paused! " + str;
 #endif
 
-#if defined(MOBILE)
+#if defined(MOBILE) && !defined(Q_OS_SYMBIAN)
     setWindowTitle(str);
 #else
     ui->lblTime->setText(str);
@@ -415,10 +480,19 @@ void MainWindow::toggleFullscreen()
         ui->menuBar->hide();
 #endif
     }
+
+    // This is what it takes to fully update the appearance
+    QApplication *app = (QApplication*) QApplication::instance();
+    app->processEvents();
+    update();
+    app->processEvents();
+    layout()->update();
+    app->processEvents();
 }
 
 void MainWindow::pause()
 {
+    qDebug() << "game paused";
     _isPaused = true;
     timer->stop();
     board->disable();
@@ -432,6 +506,7 @@ void MainWindow::pause()
 
 void MainWindow::unpause()
 {
+    qDebug() << "game resumed";
     _isPaused = false;
     timer->start();
     board->enable();
@@ -455,6 +530,7 @@ void MainWindow::togglePause()
 #elif defined (MOBILE)
             QMessageBox::information(this, "Game paused!", "You now can't move the pieces.", QMessageBox::Ok);
 #endif
+            ui->lblTime->setText("Game paused!");
         }
         else
         {
@@ -463,6 +539,7 @@ void MainWindow::togglePause()
 #elif defined (MOBILE)
             QMessageBox::information(this, "Game resumed!", "Now you can move the pieces again.", QMessageBox::Ok);
 #endif
+            updateElapsedTimeLabel();
             unpause();
         }
     }
@@ -475,6 +552,12 @@ void MainWindow::fixCurrentOrientation()
         setAttribute(Qt::WA_Maemo5LandscapeOrientation);
     else
         setAttribute(Qt::WA_Maemo5PortraitOrientation);
+#elif defined(Q_OS_SYMBIAN)
+    CAknAppUi* appUi = dynamic_cast<CAknAppUi*> (CEikonEnv::Static()->AppUi());
+    if (width() > height())
+        appUi->SetOrientationL(CAknAppUi::EAppUiOrientationLandscape);
+    else
+        appUi->SetOrientationL(CAknAppUi::EAppUiOrientationPortrait);
 #endif
 }
 
@@ -482,5 +565,14 @@ void MainWindow::unfixCurrentOrientation()
 {
 #if defined(Q_WS_MAEMO_5)
     setAttribute(Qt::WA_Maemo5AutoOrientation);
+#elif defined(Q_OS_SYMBIAN)
+    CAknAppUi* appUi = dynamic_cast<CAknAppUi*> (CEikonEnv::Static()->AppUi());
+    appUi->SetOrientationL(CAknAppUi::EAppUiOrientationAutomatic);
 #endif
 }
+
+void MainWindow::exitTriggered()
+{
+    if (QMessageBox::Yes == QMessageBox::question(this, "Are you sure?", "Do you want to exit the game?", QMessageBox::Yes, QMessageBox::No))
+        QApplication::instance()->quit();
+}
