@@ -3,7 +3,7 @@
 
 #include <QMainWindow>
 #include "settingsdialog.h"
-#include "imagechooser.h"
+#include "newgamedialog.h"
 #include "highscoresdialog.h"
 #include "puzzleboard.h"
 
@@ -19,17 +19,19 @@ class MainWindow : public QMainWindow
     Ui::MainWindow *ui;
     QTimer *timer;
     SettingsDialog *settings;
-    ImageChooser *chooser;
+    NewGameDialog *newgame;
     HighScoresDialog *highscores;
     PuzzleBoard *board;
     QPointer<QGraphicsTextItem> intro;
     bool _isPlaying;
     bool _isPaused;
     bool _wasPaused;
+    bool _canPause;
     int _secsElapsed;
     qreal _currentScaleRatio;
     void fixCurrentOrientation();
     void unfixCurrentOrientation();
+    void applyViewportSettings();
 
 protected:
     bool event(QEvent *event);
@@ -55,6 +57,7 @@ private slots:
     void updateElapsedTimeLabel();
     void initializeGame();
     void endGame();
+    void exitTriggered();
 };
 
 #endif // MAINWINDOW_H
